import java.util.*;

/** A set of locations in a Nurikabe puzzle. */
public class Region extends HashSet<Coords> {

	/** The puzzle in which this Region belongs. */
	private Nurikabe puzzle;

	/** Create an empty Region. */
	public Region(Nurikabe puzzle) {
		this.puzzle = puzzle;
	}

	/**
	 * Create a Region in puzzle containing cell and all contiguous cells with
	 * labels in allowable.
	 */
	public Region(Nurikabe puzzle, Coords cell, String allowable) {
		this.puzzle = puzzle;
		add(cell);
		find(allowable);
	}

	/** Add all contiguous cells with labels in allowable. */
	protected void find(String allowable) {
		int previousSize = -1;
		while (previousSize < size()) {
			previousSize = size();
			addAll(getNeighbors(allowable));
		}
	}

	/** Return the set of cells adjacent to this Region with labels in allowable. */
	public Region getNeighbors(String allowable) {
		Region result = new Region(puzzle);
		for (Coords cell : this) {
			for (Coords neighbor : puzzle.neighbors(cell)) {
				if (allowable.indexOf(puzzle.get(neighbor)) >= 0) {
					result.add(neighbor);
				}
			}
		}
		return result;
	}

}
